package com.fetchingStratergy.service.impl;

import org.springframework.transaction.annotation.Transactional;

import com.fetchingStratergy.domain.Employee;
import com.fetchingStratergy.service.EmployeeDetailsService;
import com.fetchingStratergy.service.EmployeeService;

public class EmployeeDetailsServiceImpl implements EmployeeDetailsService {
	
	private EmployeeService employeeService;
	
	public EmployeeDetailsServiceImpl(EmployeeService employeeService) {
		super();
		this.employeeService = employeeService;
	}

	public Employee getEmployeeDetails(int employeeId) {
		return employeeService.findEmployeeById(employeeId);
	}
	
	/**
	 * Transaction demarcation is moved from lower layer to
	 * this layer since in hibernate their is no way to tune our
	 * query for second level cached domain objects. If we eager
	 * fetch second level cached domain objects every time that
	 * data is fetched from database even though its already available from
	 * cache.
	 * employee.getDepartment() method is invoked if department object is not initialized
	 * it gets initialized else fetches from cache.
	 * If we try to fetch department object outside transaction
	 * it throws LazyInitialization exception.
	 */
	@Transactional
	public Employee getEmployeeWithDepartmentDetails(int employeeId) {
		Employee employee = employeeService.findEmployeeById(employeeId);
		/**
		 * You need to un-comment below code if empWithDepartmentDetail 
		 * point cut is comment since department data is not eagerly fetched.
		 */
		//employee.getDepartment().getName();
		return employee;
	}

	public Employee getEmployeeWithDependentDetails(int employeeId) {
		return employeeService.findEmployeeById(employeeId);
	}

}
